@@ -1,7 +1,7 @@
 #
 # This file is part of App-Cme
 #
-# This software is Copyright (c) 2014-2020 by Dominique Dumont.
+# This software is Copyright (c) 2014-2022 by Dominique Dumont <ddumont@cpan.org>.
 #
 # This is free software, licensed under:
 #
@@ -10,23 +10,29 @@
 # ABSTRACT: Run a cme script
 
 package App::Cme::Command::run ;
-$App::Cme::Command::run::VERSION = '1.032';
+$App::Cme::Command::run::VERSION = '1.037';
 use strict;
 use warnings;
-use 5.10.1;
+use v5.20;
 use File::HomeDir;
 use Path::Tiny;
 use Config::Model;
 use Log::Log4perl qw(get_logger :levels);
+use YAML::PP;
 
 use Encode qw(decode_utf8);
 
 use App::Cme -command ;
 
 use base qw/App::Cme::Common/;
+use feature qw/postderef signatures/;
+no warnings qw/experimental::postderef experimental::signatures experimental::smartmatch/;
+
 
 my $__test_home = '';
-sub _set_test_home { $__test_home = shift; }
+# used only by tests
+## no critic (Subroutines::ProhibitUnusedPrivateSubroutines)
+sub _set_test_home { $__test_home = shift; return;}
 
 my $home = $__test_home || File::HomeDir->my_home;
 
@@ -40,14 +46,13 @@ push @script_paths, path($INC{"Config/Model.pm"})->parent->child("Model/scripts"
 sub opt_spec {
     my ( $class, $app ) = @_;
     return ( 
-        [ "arg=s@"  => "script argument. run 'cme run $app -doc' for possible arguments" ],
+        [ "arg=s@"  => "script argument. run 'cme run <script> -doc' for possible arguments" ],
         [ "backup:s"  => "Create a backup of configuration files before saving." ],
         [ "commit|c:s" => "commit change with passed message" ],
         [ "cat" => "Show the script file" ],
         [ "no-commit|nc!" => "skip commit to git" ],
         [ "doc!"    => "show documention of script" ],
         [ "list!"   => "list available scripts" ],
-        [ "verbose!" => "show execution of the instructions from the 'run' script" ],
         $class->cme_global_options,
     );
 }
@@ -56,6 +61,7 @@ sub validate_args {
     my ($self, $opt, $args) = @_;
 
     $self->check_unknown_args($args);
+    return;
 }
 
 sub usage_desc {
@@ -69,16 +75,7 @@ sub description {
     return $self->get_documentation;
 }
 
-sub execute {
-    my ($self, $opt, $app_args) = @_;
-
-    # cannot use logger until Config::Model is initialised
-
-    # see Debian #839593 and perlunicook(1) section X 13
-    @$app_args = map { decode_utf8($_, 1) } @$app_args;
-
-    my $script_name = shift @$app_args;
-
+sub check_script_arguments ($self, $opt, $script_name) {
     if ($opt->{list} or not $script_name) {
         my @scripts;
         foreach my $path ( @script_paths ) {
@@ -87,11 +84,13 @@ sub execute {
         }
         say $opt->{list} ? "Available scripts:" : "Missing script argument. Choose one of:";
         say map {"- ".$_->basename."\n"} @scripts ;
-        return;
+        return 0;
     }
+    return 1;
+}
 
+sub find_script_file ($self, $script_name) {
     my $script;
-
     if ($script_name =~ m!/!) {
         $script = path($script_name);
     }
@@ -106,6 +105,197 @@ sub execute {
 
     die "Error: cannot find script $script_name\n" unless $script->is_file;
 
+    return $script;
+}
+
+# replace variables with command arguments or eval'ed variables or env variables
+## no critic (Subroutines::ProhibitManyArgs)
+sub replace_var_in_value ($user_args, $script_var, $default, $missing, $vars) {
+    my $var_pattern = qr~(?<!\\) \$([a-zA-Z]\w+) (?!\s*{)~x;
+
+    foreach ($vars->@*) {
+        # change $var but not \$var, not $var{} and not $1
+        s~ $var_pattern
+         ~ $user_args->{$1} // $script_var->{$1} // $ENV{$1} // $default->{$1} // '$'.$1 ~xeg;
+
+        # register vars without replacements
+        foreach my $var (m~ $var_pattern ~xg) {
+            $missing->{$var} = 1 ;
+        }
+
+        # now change \$var in $var
+        s!\\\$!\$!g;
+    }
+    return;
+}
+
+sub parse_script_lines ($script, $lines) {
+    # provide default values
+    my %default ;
+    my @load;
+    my @doc;
+    my @code;
+    my @var;
+    my $commit_msg ;
+    my $app;
+    my $line_nb = 0;
+
+    # check content, store app
+    while ($lines->@*) {
+        my $line = shift $lines->@*;
+        $line_nb++;
+        $line =~ s/#.*//; # remove comments
+        $line =~ s/^\s+//;
+        $line =~ s/\s+$//;
+        my ($key,@value);
+
+        if ($line =~ /^---\s*(\w+)$/) {
+            $key = $1;
+            while ($lines->[0] !~ /^---/) {
+                $lines->[0] =~ s/#.*//; # remove comments
+                push @value,  shift $lines->@*;
+            }
+        }
+        elsif ($line eq '---') {
+            next;
+        }
+        else {
+            ($key,@value) = split /[\s:]+/, $line, 2;
+        }
+
+        next unless $key ; # empty line
+
+        for ($key) {
+            when (/^app/) {
+                $app = $value[0];
+            }
+            when ('var') {
+                push @var, [ $line_nb, @value ];
+            }
+            when ('default') {
+                # multi-line default value is not supported
+                my ($dk, $dv) = split /[\s:=]+/, $value[0], 2;
+                $default{$dk} = $dv;
+            }
+            when ('code') {
+                die "Error line $line_nb: Cannot mix code and load section\n" if @load;
+                push @code, @value;
+            }
+            when ('doc') {
+                push @doc, @value;
+            }
+            when ('load') {
+                die "Error line $line_nb: Cannot mix code and load section\n" if @code;
+                push @load, @value;
+            }
+            when ('commit') {
+                $commit_msg = join "\n",@value;
+            }
+            default {
+                die "Error in file $script line $line_nb: unexpected '$key' instruction\n";
+            }
+        }
+    }
+
+    return {
+        app => $app,
+        doc => \@doc,
+        code => \@code,
+        commit_msg => $commit_msg,
+        default => \%default,
+        load => \@load,
+        var => \@var,
+    }
+}
+
+sub process_script_vars ($user_args, $data) {
+    # $var is used in eval'ed strings
+    my %var;
+
+    # find if all variables are accounted for
+    $data->{missing} = {};
+
+    # %args can be used in var section of a script. A new entry in
+    # added in %missing if the script tries to read an undefined value
+    tie my %args, 'App::Cme::Run::Var',$data->{missing}, $data->{default};
+    %args = $user_args->%*;
+
+    my $var = delete $data->{var} // [];
+    foreach my $eval_data ($var->@*) {
+        my ($line_nb, @value);
+        if (ref $eval_data) {
+            # coming from text format
+            ($line_nb, @value) = $eval_data->@*;
+            # eval'ed string comes from system file, not from user data
+            my $res = eval ("@value") ; ## no critic (ProhibitStringyEval)
+            die "Error in var specification line $line_nb: $@\n" if $@;
+        }
+        else {
+            # coming from YAML format
+            my $res = eval ($eval_data) ; ## no critic (ProhibitStringyEval)
+            die "Error in var specification: $@\n" if $@;
+        }
+    }
+
+    replace_var_in_value($user_args, \%var, $data->{default},$data->{missing}, $data->{doc});
+    replace_var_in_value($user_args, \%var, $data->{default},$data->{missing}, $data->{load});
+
+    $data->{values} = {$data->{default}->%*, %var, $user_args->%*};
+
+    return $data;
+}
+
+sub parse_script ($script, $content, $user_args) {
+    my $lines->@* =  split /\n/,$content;
+
+    given ($lines->[0]) {
+        when (/Format: perl/i) {
+            ## no critic (ProhibitStringyEval)
+            my $data = eval($content);
+            die "Error in script $script (Perl format): $@\n" if $@;
+            foreach my $forbidden (qw/load var default/) {
+                die "Unexpected '$forbidden\ section in Perl format script $script\n" if $data->{$forbidden};
+            }
+            die "Unexpected 'code' section in Perl format script $script. Please use a sub section.\n" if $data->{code};
+            return $data;
+        }
+        when (/Format: yaml/i) {
+            my $ypp = YAML::PP->new;
+            my $data = $ypp->load_string($content);
+            foreach my $key (qw/doc code load var/) {
+                next unless defined $data->{$key};
+                next if ref $data->{$key} eq 'ARRAY';
+                $data->{$key} = [ $data->{$key} ]
+            }
+            if ($data->{default} and ref $data->{default} ne 'HASH') {
+                die "default spec must be a hash ref, not a ", ref $data->{default} // 'scalar', "\n";
+            }
+            $data = process_script_vars ($user_args, $data);
+            return $data;
+        }
+        default {
+            my $data = parse_script_lines ($script, $lines);
+            $data = process_script_vars ($user_args, $data);
+            return $data;
+        }
+    }
+
+}
+
+sub execute {
+    my ($self, $opt, $app_args) = @_;
+
+    # cannot use logger until Config::Model is initialised
+
+    # see Debian #839593 and perlunicook(1) section X 13
+    @$app_args = map { decode_utf8($_, 1) } @$app_args;
+
+    my $script_name = shift @$app_args;
+
+    return unless $self->check_script_arguments($opt, $script_name);
+
+    my $script = $self->find_script_file($script_name);
+
     my $content = $script->slurp_utf8;
 
     if ($opt->{cat}) {
@@ -118,122 +308,38 @@ sub execute {
 
     if ($content =~ m/^#!/ or $content =~ /^use/m) {
         splice @ARGV, 0,2; # remove 'run script' arguments
-        eval $script->slurp_utf8; ## no critic BuiltinFunctions::ProhibitStringyEval
-        die "Error in script $script_name: $@\n" if $@;
+        my $done = eval $script->slurp_utf8."\n1;\n"; ## no critic (BuiltinFunctions::ProhibitStringyEval)
+        die "Error in script $script_name: $@\n" unless $done;
         return;
     }
 
-    my %var;
-
-    # find if all variables are accounted for
-    my %missing ;
-
-    # provide default values
-    my %default ;
-
-    # %args can be used in var section of a script. A new entry in
-    # added in %missing if the script tries to read an undefined value
-    tie my %args, 'App::Cme::Run::Var', \%missing, \%default;
-    %args = %user_args;
-
-    my $var_pattern = qr~(?<!\\) \$([a-zA-Z]\w+) (?!\s*{)~x;
-
-    # replace variables with command arguments or eval'ed variables or env variables
-    my $replace_var = sub {
-        foreach (@_) {
-            # change $var but not \$var, not $var{} and not $1
-            s~ $var_pattern
-             ~ $user_args{$1} // $var{$1} // $ENV{$1} // $default{$1} // '$'.$1 ~xeg;
-
-            # register vars without replacements
-            map { $missing{$_} = 1 ;} ( m~ $var_pattern ~xg );
-
-            # now change \$var in $var
-            s!\\\$!\$!g;
-        }
-    };
-
-    my @lines =  split /\n/,$content;
-    my @load;
-    my @doc;
-    my $commit_msg ;
-    my $line_nb = 0;
-
-    # check content, store app
-    while (@lines) {
-        my $line = shift @lines;
-        $line_nb++;
-        $line =~ s/#.*//; # remove comments
-        $line =~ s/^\s+//;
-        $line =~ s/\s+$//;
-        my ($key,@value);
-
-        if ($line =~ /^---\s*(\w+)$/) {
-            $key = $1;
-            while ($lines[0] !~ /^---/) {
-                $lines[0] =~ s/#.*//; # remove comments
-                push @value,  shift @lines;
-            }
-        }
-        elsif ($line eq '---') {
-            next;
-        }
-        else {
-            ($key,@value) = split /[\s:]+/, $line, 2;
-        }
-
-        next unless $key ; # empty line
-
-        $replace_var->(@value) unless $key eq 'var';
-
-        if ($key =~ /^app/) {
-            unshift @$app_args, @value;
-        }
-        elsif ($key eq 'var') {
-            # value comes from system file, not from user data
-            my $res = eval ("@value") ; ## no critic (ProhibitStringyEval)
-            die "Error in var specification line $line_nb: $@\n" if $@;
-        }
-        elsif ($key eq 'default') {
-            # multi-line default value is not supported
-            my ($dk, $dv) = split /[\s:=]+/, $value[0], 2;
-            $default{$dk} = $dv;
-        }
-        elsif ($key eq 'doc') {
-            push @doc, @value;
-        }
-        elsif ($key eq 'load') {
-            push @load, @value;
-        }
-        elsif ($key eq 'commit') {
-            $commit_msg = join "\n",@value;
-        }
-        else {
-            die "Error in file $script line $line_nb: unexpected '$key' instruction\n";
-        }
-    }
+    my $script_data = parse_script($script, $content, \%user_args);
+    my $commit_msg = $script_data->{commit_msg};
 
     if ($opt->doc) {
-        say join "\n", @doc;
+        say join "\n", $script_data->{doc}->@*;
         say "will commit with message: '$commit_msg'" if $commit_msg;
         return;
     }
 
-    if (my @missing = sort keys %missing) {
+    if (my @missing = sort keys $script_data->{missing}->%*) {
         die "Error: Missing variables '". join("', '",@missing)."' in command arguments for script $script\n"
             ."Please use option '".join(' ', map { "-arg $_=xxx"} @missing)."'\n";
     }
 
-    $self->process_args($opt, $app_args);
+    $self->process_args($opt, [ $script_data->{app}, $app_args->@* ]);
 
     # override commit message. may also trigger a commit even if none
     # is specified in script
-    $commit_msg ||= $opt->{commit};
+    if ($opt->{commit}) {
+        $commit_msg = $opt->{commit};
+    }
 
     # check if workspace and index are clean
-    if ($commit_msg) {
+    if ($commit_msg and not $opt->{no_commit}) {
+        ## no critic(InputOutput::ProhibitBacktickOperators)
         my $r = `git status --porcelain --untracked-files=no`;
-        die "Cannot run commit command in a non clean repo. Please commit or stash pending changes: $r"
+        die "Cannot run commit command in a non clean repo. Please commit or stash pending changes: $r\n"
             if $r;
     }
 
@@ -241,20 +347,44 @@ sub execute {
 
     # call loads
     my ($model, $inst, $root) = $self->init_cme($opt,$app_args);
-    map { $root->load($_) } @load;
+    foreach my $load_str ($script_data->{load}->@*) {
+        $root->load($load_str);
+    }
+
+    if ($script_data->{code}) {
+        my $to_run = '';
+        while (my ($name, $value) = each $script_data->{values}->%*) {
+            $to_run .= "my \$$name = '$value';\n";
+        }
+        $to_run .= join("\n",$script_data->{code}->@*);
+        my $res = eval($to_run); ## no critic (ProhibitStringyEval)
+        die "Error in code specification: $@\ncode is: \n$to_run\n" if $@;
+    }
+
+    if ($script_data->{sub}) {
+        $script_data->{sub}->($root, \%user_args);
+    }
+
+    unless ($inst->needs_save) {
+        say "No change were applied";
+        return;
+    }
 
     $self->save($inst,$opt) ;
 
     # commit if needed
-    if ($commit_msg and not $opt->{'no-commit'}) {
+    if ($commit_msg and not $opt->{no_commit}) {
         system(qw/git commit -a -m/, $commit_msg);
     }
+
+    return;
 }
 
-package App::Cme::Run::Var;
-$App::Cme::Run::Var::VERSION = '1.032';
+package App::Cme::Run::Var; ## no critic (Modules::ProhibitMultiplePackages)
+$App::Cme::Run::Var::VERSION = '1.037';
 require Tie::Hash;
 
+## no critic (ClassHierarchies::ProhibitExplicitISA)
 our @ISA = qw(Tie::ExtraHash);
 
 sub FETCH {
@@ -279,7 +409,7 @@ App::Cme::Command::run - Run a cme script
 
 =head1 VERSION
 
-version 1.032
+version 1.037
 
 =head1 SYNOPSIS
 
@@ -318,8 +448,7 @@ version 1.032
 
 =head1 DESCRIPTION
 
-Run a script written with cme DSL (Design specific language) or in
-plain Perl.
+Run a script written for C<cme>
 
 A script passed by name is searched in C<~/.cme/scripts>,
 C</etc/cme/scripts> or C</usr/share/perl5/Config/Model/scripts>.
@@ -330,10 +459,35 @@ C</usr/share/perl5/Config/Model/scripts/foo>
 No search is done if the script is passed with a path
 (e.g. C<cme run ./foo>)
 
+C<cme run> accepts scripts written with different syntaxes:
+
+=over
+
+=item in text
+
+For simple script, this text specifies the target app, the doc,
+optional variables and a load string used by L<Config::Model::Loader> or
+Perl code.
+
+=item YAML
+
+Like text above, but using Yaml syntax.
+
+=item Perl data structure
+
+Writing Perl code in a text file or in a YAML field can be painful as
+Perl syntax is not highlighted. With a Perl data structure, a cme
+script specifies the target app, the doc, optional variables, and a
+perl subroutine (see below).
+
+=item plain Perl script
+
 C<cme run> can also run plain Perl script. This is syntactic sugar to
 avoid polluting global namespace, i.e. there's no need to store a
 script using L<cme function|Config::Model/cme> in C</usr/local/bin/>.
 
+=back
+
 When run, this script:
 
 =over
@@ -344,7 +498,7 @@ opens the configuration file of C<app>
 
 =item *
 
-applies the modifications specified with C<load> instructions
+applies the modifications specified with C<load> instructions or the Perl code.
 
 =item *
 
@@ -358,7 +512,7 @@ commits the result if C<commit> is specified (either in script or on command lin
 
 See L<App::Cme::Command::run> for details.
 
-=head1 Syntax
+=head1 Syntax of text format
 
 The script accepts instructions in the form:
 
@@ -382,6 +536,14 @@ Specify the target application. Must be one of the application listed
 by C<cme list> command. Mandatory. Only one C<app> instruction is
 allowed.
 
+=item default
+
+Specify default values that can be used in C<load> or C<var> sections.
+
+For instance:
+
+ default: name=foobar
+
 =item var
 
 Use Perl code to specify variables usable in this script. The Perl
@@ -405,6 +567,10 @@ L<Config::Model::Loader>. This string can contain variable
 foo=bar>) or by a variable set in var: line (e.g. C<$var{foo}> as set
 above) or by an environment variable (e.g. C<$ENV{foo}>)
 
+=item code
+
+Specify Perl code to run. See L</code section> for details.
+
 =item commit
 
 Specify that the change must be committed with the passed commit
@@ -429,6 +595,91 @@ in
 
   load: ! a=foo b=bar
 
+=head2 Example
+
+Here's an example from L<libconfig-model-dpkg-perl scripts|https://salsa.debian.org/perl-team/modules/packages/libconfig-model-dpkg-perl/-/blob/master/lib/Config/Model/scripts/add-me-to-uploaders>:
+
+  doc: add myself to Uploaders
+  app: dpkg-control
+  load: source Uploaders:.insort("$DEBFULLNAME <$DEBEMAIL>")
+  commit: add $DEBEMAIL to Uploaders
+
+=head2 Code section
+
+The code section can contain variable (e.g. C<$foo>) which are replaced by
+command argument (e.g. C<-arg foo=bar>) or by a variable set in var:
+line (e.g. C<$var{foo}> as set above).
+
+When evaluated the following variables are also set:
+
+=over
+
+=item $root
+
+Root node of the configuration (See L<Config::Model::Node>)
+
+=item $inst
+
+Configuration instance (See L<Config::Model::Instance>)
+
+=item $commit_msg
+
+Message used to commit the modification.
+
+=back
+
+Since the code is run in an C<eval>, other variables are available
+(like C<$self>) to shoot yourself in the foot.
+
+For example:
+
+ app:  popcon
+ ---code
+ $root->fetch_element('MY_HOSTID')->store($to_store);
+ ---
+
+=head1 Syntax of YAML format
+
+This format is intented for people not wanting to user the text format
+above. It supoorts the same parameters as the text format.
+
+For instance:
+
+ # Format: YAML
+ ---
+ app: popcon
+ default:
+   defname: foobar
+ var: "$var{name} = $args{defname}"
+ load: "! MY_HOSTID=$name"
+
+=head1 Syntax of Perl format
+
+This format is intended for more complex script where using C<load>
+instructions is not enough.
+
+This script must then begin with C<# Format: perl> and specifies a
+hash. For instance:
+
+ # Format: perl
+ {
+      app => 'popcon', # mandatory
+      doc => "Use --arg to_store=a_value to store a_value in MY_HOSTID',
+      commit => "control: update Vcs-Browser and Vcs-Git"
+      sub => sub ($root, $arg) { $root->fetch_element('MY_HOSTID')->store($arg->{to_store}); }
+ }
+
+C<$root> is the root if the configuration tree (See L<Config::Model::Node>).
+C<$arg> is a hash containing the arguments passed to C<cme run> with C<-arg> options.
+
+The C<sub> parameter value must be a sub ref. Its parameters are
+C<$root> (a L<Config::Model::Node> object containing the root of the
+configuration tree) and C<$arg> (a hash ref containing the keys and
+values passed to C<cme run> wiht C<--arg> options).
+
+Note that this format does not support C<var>, C<default> and C<load>
+parameters as you can easily achieve the same result with Perl code.
+
 =head1 Options
 
 =head2 list
@@ -545,7 +796,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014-2020 by Dominique Dumont.
+This software is Copyright (c) 2014-2022 by Dominique Dumont <ddumont@cpan.org>.
 
 This is free software, licensed under:
 
