@@ -1,7 +1,7 @@
 #
 # This file is part of App-Cme
 #
-# This software is Copyright (c) 2014-2020 by Dominique Dumont.
+# This software is Copyright (c) 2014-2022 by Dominique Dumont <ddumont@cpan.org>.
 #
 # This is free software, licensed under:
 #
@@ -10,7 +10,7 @@
 #ABSTRACT: Common methods for App::Cme
 
 package App::Cme::Common;
-$App::Cme::Common::VERSION = '1.032';
+$App::Cme::Common::VERSION = '1.037';
 use strict;
 use warnings;
 use 5.10.1;
@@ -25,8 +25,6 @@ use Encode qw(decode_utf8);
 
 my @store;
 
-$::_use_log4perl_to_warn = 1;
-
 sub cme_global_options {
   my ( $class, $app ) = @_;
 
@@ -35,7 +33,7 @@ sub cme_global_options {
       [ "try-app-as-model!"  => "try to load a model using directly the application name "
                               . "specified as 3rd parameter on the command line"],
       [ "save!"              => "Force a save even if no change was done" ],
-      [ "force-load!"        => "Load file even if error are found in data. Bad data are discarded (imply save)"],
+      [ "force-load!"        => "Load file even if error are found in data. Errors must be fixed before saving."],
       [ "create!"            => "start from scratch."],
       [ "root-dir=s"         => "Change root directory. Mostly used for test"],
       [ "file=s"             => "Specify a target file"],
@@ -58,7 +56,8 @@ sub check_unknown_args {
 
     my @unknown_options = grep { /^-/ } @$args ;
     # $self->usage_error("Unknown option: @unknown_options") if @unknown_options;
-    warn("Unknown option: @unknown_options. Unknown option will soon be a fatal error.") if @unknown_options;
+    warn("Unknown option: @unknown_options. Unknown option will soon be a fatal error.\n") if @unknown_options;
+    return;
 }
 
 # modifies $args in place
@@ -117,6 +116,10 @@ sub process_args {
             die "application $application requires a 3rd argument$insert. "
                 . "I.e. 'cme $command $application <backend_arg>'\n";
         }
+
+        if ( $appli_info->{$application}{use_backend_argument_as_config_file} ) {
+            $config_file = $appli_info->{$application}{config_dir} . '/' . $b_arg;
+        }
     }
 
     # remove legacy '~~'
@@ -132,8 +135,9 @@ sub process_args {
     $opt->{_application} = $application ;
     $opt->{_config_file} = $config_file;
     $opt->{_root_model}  = $root_model;
-}
 
+    return;
+}
 
 sub model {
     my ($self, $opt, $args) = @_;
@@ -172,9 +176,9 @@ sub instance {
 }
 
 sub init_cme {
-    my $self = shift;
+    my ($self, @args) = @_;
     # model and inst are deleted if not kept in a scope
-    return ( $self->model(@_) , $self->instance(@_), $self->instance->config_root );
+    return ( $self->model(@args) , $self->instance(@args), $self->instance->config_root );
 }
 
 sub save {
@@ -186,6 +190,7 @@ sub save {
     # at semantic level, i.e. removed unnecessary stuff)
     $inst->write_back( force => $opt->{force_load} || $opt->{save} );
 
+    return;
 }
 
 sub run_tk_ui {
@@ -196,7 +201,7 @@ sub run_tk_ui {
     require Tk::ErrorDialog;
     Tk->import;
 
-    no warnings 'once';
+    no warnings 'once'; ## no critic (TestingAndDebugging::ProhibitNoWarnings)
     my $mw = MainWindow->new;
     $mw->withdraw;
 
@@ -214,6 +219,8 @@ sub run_tk_ui {
     }
 
     &MainLoop;    # Tk's
+
+    return;
 }
 
 sub run_shell_ui ($$$) {
@@ -227,6 +234,8 @@ sub run_shell_ui ($$$) {
 
     # engage in user interaction
     $shell_ui->run_loop;
+
+    return;
 }
 
 sub get_documentation {
@@ -236,7 +245,7 @@ sub get_documentation {
     my $pkg = blessed ($self);
     $pkg =~ s!::!/!g;
     my $pom = $parser->parse_file($INC{$pkg.'.pm'})
-        || die $parser->error();
+        || croak $parser->error();
 
     my $sections = $pom->head1();
     my @ret ;
@@ -260,7 +269,7 @@ App::Cme::Common - Common methods for App::Cme
 
 =head1 VERSION
 
-version 1.032
+version 1.037
 
 =head1 SYNOPSIS
 
@@ -276,7 +285,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014-2020 by Dominique Dumont.
+This software is Copyright (c) 2014-2022 by Dominique Dumont <ddumont@cpan.org>.
 
 This is free software, licensed under:
 
